"""Grid Configuration Monitoring Client WN

This module starts the Worker Node Client for Grid Monitoring.

Author: Thomas Low <thomas.low@cern.ch>
"""

import os, sys

gcm_home = os.environ.get('GCM_HOME','/opt/glite')
os.environ['GCM_HOME'] = gcm_home
sys.path.append(gcm_home+'/lib/python')

import getopt, logging
log = logging.getLogger(__name__)

from gcm.common import logger, config, event
from gcm.common.testmanager import TestManager


def run():
	"""Start WN Client"""
	conf = _loadConfig()
	logger.setupColoredStreamLogging(**config.getSectionAsDict(conf,'logger'))
	#logger.setupFileLogging('/afs/cern.ch/user/t/tlow/log/grid-cm-client-wn.error.log', level=logging.ERROR)
	#logger.setupFileLogging('/afs/cern.ch/user/t/tlow/log/grid-cm-client-wn.debug.log', level=logging.DEBUG)
	log.debug("start")
	Client().run()
	log.debug("end")
	

class Client(TestManager):
	"""Abstracts a simple client which can just be started by run().
	
	It loads a two types of components (consumer and producer) dynamically from
	the configuration file and runs their corespronding methods. (setup, 
	run/consume, cleanup).
	"""
	
	def __init__(self):
		"""Inits the Client.
		
		Here all modules will be loaded from the configuration file and stored
		in two lists self.consumer, self.producer.
		"""
		TestManager.__init__(self)
		self.conf = _loadConfig()
		self.errors = []
		self.event.append(self._printTest)
				
		#load consumer from configuration file and link them dynamically
		for c in self.conf.get('client','consumer').split(" "):
			if c.strip():
				args = {}
				if self.conf.has_section(c):
					args = config.getSectionAsDict(self.conf,c)
				self.addConsumer(c,args)
		
		#load producers from configuration file and link them dynamically
		for p in self.conf.get('client','producer').split(" "):
			if p.strip():
				args = {}
				if self.conf.has_section(p):
					args = config.getSectionAsDict(self.conf,p)
				self.addProducer(p, args)
	
		
	def _printTest(self, test): #just for debugging
		"""Prints a test to stdout for debugging."""
		log.info("Test \"%s\" in Eventloop" % test.testName)
		
	def run(self):
		"""Executes setup, run and finally cleanup."""
		self.doSetup()
		self.doRun()
		self.doCleanup()
		

def _loadConfig():
	"""Loads the configuration file of the client into a variable."""
	opts, _ = _parseArguments()
	conf = config.load('grid-cm-client-wn.conf', 'GCM_HOME', 
							opts.get('c'),'GCM_CLIENTS_WN_CONFIG')		
	if opts.has_key('v'):
		conf.set('logger','level','10')
	return conf


def _parseArguments():
	"""Parses some basic arguments."""
	try:
		opts, args = getopt.getopt(sys.argv[1:],
			"hvc:", ["help", "verbose", "config="])
	except getopt.GetoptError, err:
		_printUsage(str(err))
		sys.exit(2)
	options = {}
	for (o,a) in opts:
		options[o.replace('-','')[0]] = a
	if args:
		_printUsage("no arguments allowed")
		sys.exit(2)
	if 'h' in options.keys():
		_printUsage()
		sys.exit(0)
	return options, args


def _printUsage(error=''):
	"""Prints the Usage if necessary."""
	d = { 	'error' : error, 
			'name'  : os.path.basename(sys.argv[0]),
			'w' 	: '\033[1;37m', #set console text to white
			'ul'	: '\033[4m', #underline console text
			'n'		: '\033[m' } #reset console formatting
	if error:
		print "%(w)sERROR:%(n)s\t %(error)s\n" % d
	print """%(w)sNAME:%(n)s
\t %(name)s - Grid Configuration Monitoring: Client Worker Node

%(w)sSYNOPSIS: %(n)s
\t %(name)s [-hv] [-c %(ul)sconfig_file%(n)s]

%(w)sOPTIONS:%(n)s
\t -h, --help
\t\t Print this help text and exit.
\t -v, --verbose
\t\t Be more verbose
\t -c, --config %(ul)spath%(n)s
\t\t Use alternative path to configuration file for %(name)s
""" % d


if __name__ == '__main__':
	run()
	
	
