"""Pyamq Sender Class

This module provides a Consumer Class which sends Tests via PyAMQ.

Author: Thomas Low <thomas.low@cern.ch>
"""

import time, logging, re
log = logging.getLogger(__name__)

from gcm.common.testconsumer import TestConsumer
from gcm.common.test import Test
from gcm.common import openssl, test

from pyamq import producer


class PyamqSender(TestConsumer):
	"""This module converts test results to a activeMQ message and sends it."""
	
	def __init__(self, 	broker, 
						destination, 
						connections	= 1, 
						random		= 0, 
						timeout		= None, 
						pubkey		= None,
						testfilter	= None):
		"""Inits the Sender with a set of arguments.
		
		BROKER space seperated string of connection-strings
		DESTINATION destination where to send the messages
		CONNECTIONS number of multiple connections
		RANDOM if true, list of brokers will be processed randomly
		TIMEOUT maximum timeout the sender will run after start
		PUBKEY public key which will be used to encrypt all messages
		TESTFILTER a regular expression for the test name
		"""
		
		self.broker = broker.split(" ")
		self.destination = destination
		self.connections = int(connections)
		self.random = int(random)
		self.timeout = float(timeout)
		self.endtime = time.time()+self.timeout
		self.pubkey = pubkey
		if testfilter:
			self.testpattern = re.compile(testfilter)
		else:
			self.testpattern = None
		self.pyamqid = test._getUniqueId()
		
	def setup(self):
		self.producer = producer.Producer(self.broker, self.connections, 
											self.random)
		
	def consume(self, test):
		"""Sends a test."""		
		if self.testpattern and not self.testpattern.match(test.testName):
			log.debug("PyamqSender: \"%s\" skipped" % test.testName)
			return
		test.pyamqId = self.pyamqid
		msg = str(test)
		
		if test.has_key("pyamqEncrypt") and test.pyamqEncrypt == '1':
			msg = openssl.encrypt(msg, self.pubkey)
			log.debug("PyamqSender: \"%s\" encrypted" % test.testName)
		
		self.producer.sendMessage({
			'header'		: {'destination': self.destination},
			'body'		 	: msg
		})
		
	def cleanup(self):
		self.producer.wait(self.endtime-time.time())
		p,s = self.producer.stop()
		if p:
			exitcode = 2
			log.warning("PyamqSender: %d out of %d Message not sent" % (len(p),len(p+s)))
