"""Process Producer Class

This module provides a Process-Producer Class to create new tests.

ProcTester executes a set of executables within a directory and creates tests
from their stdout. SignedProcTester uses a signed tar ball to do the same.

Author: Thomas Low <thomas.low@cern.ch>
"""

import sys, os, time, logging, random, os.path, tempfile
log = logging.getLogger(__name__)

from gcm.common.testproducer import TestProducer
from gcm.common import test, openssl, tar
from gcm.common.test import Test
from pyamq.lib import processmgr

from gcm.clients.wn import util

class ProcTester(TestProducer):
	"""This Producer will call a set of executables in directory.
	
	Therefore it lookes for a filename for which an executable file and a file
	filename.def exists. The tuples within filename.def will be loaded into a
	test. Then the file will be executed and the stdout will be merged into the
	test together with some basic information like starttime, endtime, exitcode.
	The stderr will be redirected to the data-field of the test.
	"""
	def __init__(self, 
					directory,
					timeout = None,
					threads = 3 ):
		"""Just saves the arguments and does some initiating."""
		self.directory = directory
		self.timeout = float(timeout)
		self.endtime = time.time() + self.timeout
		self.threads = int(threads)
		self.tests = {}
		self.event = None
		
	def setup(self):
		"""Loads the test from the directory.
		
		Merges the *.def files with a Test-Class and stores them in self.tests
		"""
		#create list of testfiles
		pathlist = []
		for f in os.listdir(self.directory):
			path = os.path.join(self.directory,f)
			if f[-8:] == '.testdef' and os.access(path, os.F_OK) \
					and not os.access(path, os.X_OK):
				pathlist.append(path)
					
		#create test for each testfile		
		for path in pathlist:
			try:
				t = self._createTestFromFile(path)
				if t:
					self.tests[t.executionCommand] = t
			except Exception, err:
				log.error("%s : %s" % (os.path.basename(path), err))
		
	
	def _createTestFromFile(self, path):
		lines = open(path, 'r').readlines()
		t = test.stringToTest("".join(lines))
		
		#setup command to execute
		if t.has_key('executionFile'):
			pathExec = os.path.dirname(path)+'/'+t.executionFile
		else:
			#try to guess executionFile
			pathExec = os.path.dirname(path)+'/'+os.path.basename(path).replace('.testdef','')
		if not os.access(pathExec, os.X_OK):
			raise ValueError("executionFile is not executable: %s" % pathExec)
		t.executionCommand = pathExec	
		if t.has_key('executionArguments'):
			t.executionCommand += t.executionArguments	
		
		#compute execution probability
		if t.has_key('executionProbability'):
			prob = float(t.executionProbability)
			if prob < 0 or prob > 1:
				raise ValueError("value not between 0 and 1")
			rand = random.random()
			if prob < rand:
				log.debug("%s lost game (%.3f to %.3f)" % \
					(os.path.basename(t.executionCommand), prob, rand))
				return None
				
		
		return t 
		
		
	def _convertProcToTest(self, cmd, data):
		"""Gets results of a subprocess and converts them into a test-class."""
		#find test with certain command
		
		t = self.tests.get(cmd)				
		if not t:
			log.error("process could not be assigned to a test")
			return
	
		#setup some basic attributes
		t.hostname	= util.hostname
		t.hostid	= util.hostid
		t.executionUnit = "ProcTester"
		t.executionExitcode	= str(data['exitcode'])
		t.executionExitsignal= str(data['signal'])
		t.executionStarttime = repr(data['starttime'])
		t.executionEndtime 	= repr(data['endtime'])
		
		#convert stdout of test into arguments for the test-class
		errors = []
		lines = data['stdout'].split('\n')
		for line in lines:
			if line == '':
				continue
			kv = line.split(':',1)
			if len(kv) == 2:
				try:
					setattr(t,kv[0].strip(),kv[1].strip())
				except ValueError, err:
					log.error("[%s] ValueError: %s (key: %s, value: %s)" % \
						(t.testName, err, repr(kv[0]), repr(kv[1])))
					errors.append("ValueError: %s (key: %s, value: %s)" % \
						(t.testName, err, repr(kv[0]), repr(kv[1])))
	
		if data['stderr']:
			t.data = str(data['stderr'])
		if errors:
			if not t.has_key('data'):
				t.data = ''
			t.data += 'Format Errors:\n'+'\n'.join(errors)
		
		log.warning("%s needed %.5f seconds" % (t.testName, time.time()-float(data['starttime'])))
		
		self.event(t)
	
	
	def run(self, event):
		"""Runs all loaded tests with their specified timeout."""
		self.event = event
		if self.tests:
			self.testMgr = processmgr.ProcessManager(self.threads)
			for t in self.tests.values():
				try:
					timeout = min(float(t.executionTimeout),self.endtime-time.time())
				except:
					timeout = self.endtime-time.time()
				self.testMgr.execute(t.executionCommand, self._convertProcToTest, timeout)
			
	
	def cleanup(self):
		"""Waits until all tests are done."""
		if self.tests:
			self.testMgr.wait(self.endtime-time.time())
			self.testMgr.stop()

		
class SignedProcTester(ProcTester):

	def __init__(self, 
					directory,
					certs,
					allowed,
					timeout = None,
					threads = 3):
		"""Just saves the arguments and does some initiating."""
		self.certs = certs
		self.allowed = allowed.split(':')
		self.tmpDirs = []
		ProcTester.__init__(self, directory, timeout, threads)

	def setup(self):
		"""Looks for tar balls, extracts the files and creates test-classes."""
		pathlist = []
		for f in os.listdir(self.directory):
			path = os.path.join(self.directory,f)
			if f[-10:] == '.tar.smime' and os.access(path, os.F_OK) \
					and not os.access(path, os.X_OK):
				pathlist.append(path)
				
		for path in pathlist:
			#try to verify tar ball
			tarFile = tempfile.mktemp()
			tarDir  = openssl._mkdtemp(self.directory)
			self.tmpDirs.append(tarDir)
			try:
				cert = openssl.verifyFile(path, tarFile, self.certs)
				#find out subject and check against allowed
				subj = openssl.x509subject(cert)
				if subj in self.allowed:
					#extract tar ball to tempdirectory
					tar.extract(tarFile, tarDir)
					#look for *.def files in tarDir
					for f in os.listdir(tarDir):
						tarDirPath = os.path.join(tarDir,f)
						if tarDirPath[-8:] == '.testdef' \
								and os.access(tarDirPath, os.F_OK) \
								and not os.access(tarDirPath, os.X_OK):
							t = self._createTestFromFile(tarDirPath)
							self.tests[t.executionCommand] = t
				else:
					raise Exception("signer is not allowed: %s" % subj)
			except Exception, err:
				log.error("%s : %s" % (os.path.basename(path), err))
			try: os.remove(tarFile)
			except: pass
			
	def cleanup(self):
		ProcTester.cleanup(self)
		for t in self.tmpDirs:
			try:
				os.system('rm -rf  %s ' % t)
			except Exception, err:
				log.warning(err)
